package ru.sicamp.sicamphelper.api.command

import kotlinx.coroutines.runBlocking
import mu.KLogging
import org.telegram.telegrambots.extensions.bots.commandbot.commands.BotCommand
import org.telegram.telegrambots.meta.api.methods.send.SendMessage
import org.telegram.telegrambots.meta.api.objects.Chat
import org.telegram.telegrambots.meta.api.objects.User
import org.telegram.telegrambots.meta.bots.AbsSender
import ru.sicamp.sicamphelper.model.metadata.RequestSource
import ru.sicamp.sicamphelper.db.table.Users
import ru.sicamp.sicamphelper.model.request.Request
import ru.sicamp.sicamphelper.model.request.RequestWrapper
import ru.sicamp.sicamphelper.model.response.Response
import ru.sicamp.sicamphelper.model.response.ResponseWrapper
import ru.sicamp.sicamphelper.model.response.TelegramResponse
import ru.sicamp.sicamphelper.service.UserService
import ru.sicamp.sicamphelper.util.Extractor

abstract class Command<REQ : Request, RES : TelegramResponse>(
    private val command: Commands
) : BotCommand(command.name, command.description), Extractor<REQ> {
    protected abstract val userService: UserService

    protected abstract suspend fun executeRequest(request: RequestWrapper<REQ>): ResponseWrapper<RES>

    override fun execute(absSender: AbsSender?, user: User?, chat: Chat?, arguments: Array<out String>?) {
        val internalUser = authorizeCommand(user) ?: TODO("Respond unauthorized request")
        val request = try {
            RequestWrapper(
                issuer = internalUser,
                source = RequestSource.TELEGRAM,
                body = extract(arguments)
            )
        } catch (e: Exception) {
            TODO("Respond error")
        }
        try {
            val response = runBlocking {
                executeRequest(request)
            }
            response.getTelegramMessage()?.let {
                absSender?.execute(it)
                TODO("Respond success")
            }
        } catch (e: Exception) {
            logger.error { e }
            TODO("Respond internal error")
        }
    }

    private fun authorizeCommand(user: User?): ru.sicamp.sicamphelper.db.entity.User? = userService.findUserByTgUser(
        user ?: error("No user provided for command $commandIdentifier")
    ).let {
        if (it == null) {
            logger.warn("Not found user with tgUsername: ${user.userName}, tgId: ${user.id}")
            return@let null
        }
        if (it.role !in Users.Role.upperRoles(command.minimalRole)) {
            logger.warn("User $it is not authorized to perform command $commandIdentifier")
            return@let null
        }
        it
    }

    companion object : KLogging()
}