package ru.sicamp.sicamphelper.db.table

import org.jetbrains.exposed.dao.id.LongIdTable
import org.jetbrains.exposed.sql.javatime.datetime

object Users : LongIdTable("users") {
    val name = varchar("name", 255)
    val supervisor = reference("supervisor", Users).nullable()
    val role = enumerationByName<Role>("role", 255)
    val group = reference("group", Groups)
    val tgId = long("tg_id").uniqueIndex("tg_id_index").nullable()
    val tgUsername = varchar("tg_username", 255).nullable()
    val sex = enumerationByName<Sex>("sex", 255)
    val age = integer("age")

    val login = varchar("login", 255)
    val password = varchar("password", 255)
    val enabled = bool("enabled")
    val locked = bool("locked")
    val accountExpiration = datetime("account_expiration")
    val credentialsExpiration = datetime("credentials_expiration")
    //TODO add photos

    enum class Role(val level: Int) { STUDENT(0), TEACHER(1), SENIOR_TEACHER(2), PRINCIPAL(3), ADMIN(4);
        companion object {
            fun upperRoles(role: Role): List<Role> = values().filter { it.level <= role.level }
            fun lowerRoles(role: Role): List<Role> = values().filter { it.level >= role.level }
        }
    }
    enum class Sex { MALE, FEMALE, OTHER }
}