package ru.sicamp.sicamphelper.helper

import org.jetbrains.exposed.dao.with
import org.springframework.stereotype.Component
import ru.sicamp.sicamphelper.ConfigNames
import ru.sicamp.sicamphelper.db.entity.Event
import ru.sicamp.sicamphelper.db.entity.Room
import ru.sicamp.sicamphelper.db.entity.Schedule
import ru.sicamp.sicamphelper.db.entity.User
import ru.sicamp.sicamphelper.service.ConfigService
import ru.sicamp.sicamphelper.util.InTransaction

@Component
class ScheduleHelper(
    private val configService: ConfigService
) {
    @InTransaction
    fun updateSchedulesByEvent(newEvent: Event, oldEvent: Event? = null): List<Schedule> {
        val oldScheduleUpdates = oldEvent?.let { old ->
            old.schedules.notForUpdate().mapIndexedNotNull { index, schedule ->
                ScheduleUpdate(
                    description = schedule.description,
                    responsible = schedule.responsible.takeIf { schedule.responsible != old.responsible },
                    room = schedule.room.takeIf { schedule.room != old.defaultRoom }
                ).takeIf { !it.isEmpty() }?.let {
                    index to it
                }
            }.toMap()
        }

        val startDate = newEvent.eventDate ?: configService.getDate(ConfigNames.CAMP_START)
        val dates = generateSequence(seed = startDate) { prevDate ->
            val nextDate = prevDate + newEvent.repeat
            if (nextDate <= configService.getDate(ConfigNames.CAMP_END)) {
                nextDate
            } else {
                null
            }
        }
        return dates.mapIndexed { index, date ->
            val scheduleUpdate = oldScheduleUpdates?.get(index)
            Schedule.new {
                dateTime = date.atTime(newEvent.eventTime)
                responsible = scheduleUpdate?.responsible ?: newEvent.responsible
                event = newEvent
                room = scheduleUpdate?.room ?: newEvent.defaultRoom
            }
        }
            .toList()
            .with(
                Schedule::event,
                Schedule::responsible,
                Schedule::room
            )
    }

    private data class ScheduleUpdate(
        val description: String?,
        val responsible: User?,
        val room: Room?
    ) {
        fun isEmpty() = description != null || responsible != null || room != null
    }
}