package ru.sicamp.sicamphelper.model.request

import ru.sicamp.sicamphelper.model.metadata.Cursor
import ru.sicamp.sicamphelper.model.metadata.Error
import ru.sicamp.sicamphelper.model.metadata.MetaData
import ru.sicamp.sicamphelper.model.metadata.RequestSource
import ru.sicamp.sicamphelper.model.response.Response
import ru.sicamp.sicamphelper.model.response.ResponseWrapper
import ru.sicamp.sicamphelper.db.entity.User

class RequestWrapper<REQ : Request>(
    val issuer: User,
    val source: RequestSource,
    val cursor: Cursor? = null,
    val body: REQ
) {
    fun <RES : Response> success(response: RES, newCursor: Cursor? = null): ResponseWrapper<RES> =
        ResponseWrapper(
            issuer = issuer,
            source = source,
            metaData = MetaData(
                cursor = newCursor,
                error = null
            ),
            response = response
        )

    fun <RES : Response> fail(statusCode: Int = 500, message: String = "Unknown Error"): ResponseWrapper<RES> =
        ResponseWrapper(
            issuer = issuer,
            source = source,
            metaData = MetaData(
                error = Error(statusCode, message),
                cursor = cursor,
            ),
            response = null
        )
}