package ru.sicamp.sicamphelper.service

import org.jetbrains.exposed.sql.transactions.transaction
import org.springframework.stereotype.Service
import ru.sicamp.sicamphelper.db.entity.User
import ru.sicamp.sicamphelper.db.table.Users
import ru.sicamp.sicamphelper.model.data.UserInput
import ru.sicamp.sicamphelper.model.request.RequestWrapper
import ru.sicamp.sicamphelper.model.request.info.InfoRequest
import ru.sicamp.sicamphelper.model.response.ResponseWrapper
import ru.sicamp.sicamphelper.model.response.info.InfoResponse

@Service
class InfoService {
    suspend fun getInfo(requestWrapper: RequestWrapper<InfoRequest>): ResponseWrapper<InfoResponse> {
        val request = requestWrapper.body
        val userInput = request.userInput
        val type = userInput?.type
        val user = transaction {
            when (type) {
                UserInput.Type.USER_ID -> User.findById(userInput.userId ?: return@transaction null)
                UserInput.Type.TG_USERNAME -> User.find { Users.tgUsername eq userInput.tgUsername }.firstOrNull()
                UserInput.Type.GATE_LOGIN -> userInput.gateLogin?.let { User.find { Users.login eq it }.firstOrNull() }
                null -> requestWrapper.issuer
            }
        }

        return if (user != null) {
            requestWrapper.success(InfoResponse(user))
        } else {
            requestWrapper.fail(statusCode = 404, message = "Not found user")
        }
    }
}